#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <windows.h>
#include <mapi.h>

#include "wine/debug.h"

WINE_DEFAULT_DEBUG_CHANNEL(mapi);

static void usage(void)
{
    printf("Usage: mapitest [--to=<emails>] [--bcc=<emails> ] [--body=<body> ] [--subject=<subject>] [--file=<file> ]\n");
}

char *to = NULL;
char *bcc = NULL;
char *subject = NULL;
char *file = NULL;
char *body = NULL;

int parse_arguments(int argc, char *argv[])
{
    int rc;
    int longindex = 0;

    enum option_types {  OPTION_TO, OPTION_BCC, OPTION_SUBJECT, OPTION_BODY, OPTION_FILE, OPTION_HELP };

    static struct option long_options[] =
    {
        {"to",          1, 0, OPTION_TO         },
        {"bcc",         1, 0, OPTION_BCC        },
        {"subject",     1, 0, OPTION_SUBJECT    },
        {"body",        1, 0, OPTION_BODY       },
        {"file",        1, 0, OPTION_FILE       },
        {"help",        0, 0, OPTION_HELP       },
        {0, 0, 0, 0}
    };

    while (1) {
        rc = getopt_long_only(argc, argv, "", long_options, &longindex);
        if (rc == -1) {
            rc = 0;
            break;
        }

        switch (rc) {
            case OPTION_TO:
                to = strdup(optarg);
                break;

            case OPTION_BCC:
                bcc = strdup(optarg);
                break;

            case OPTION_SUBJECT:
                subject = strdup(optarg);
                break;

            case OPTION_BODY:
                body = strdup(optarg);
                break;

            case OPTION_FILE:
                file = strdup(optarg);
                break;

            default:
                usage();
                return -1;
        }
    }

    if (optind < argc)
    {
        fprintf(stderr, "Error: too many arguments\n");
        return -1;
    }

    return 0;
}

int main(int argc, char** argv)
{
    HMODULE hlib;
    int rc;
    char *saveptr;
    LPMAPISENDMAIL mailer;
    MapiMessage message;
    MapiFileDesc file_desc;
    MapiRecipDesc originator;

    rc = parse_arguments(argc, argv);
    if (rc)
        exit(rc);


    hlib = LoadLibraryA("MAPI32.DLL");
    if (! hlib)
    {
        fprintf(stderr, "Error: unable to LoadLibrary MAPI32.DLL\n");
        exit(1);
    }

    mailer = (LPMAPISENDMAIL) GetProcAddress(hlib, "MAPISendMail");
    if (!mailer)
    {
        fprintf(stderr, "Error: unable to GetProcAddress MAPISendMail\n");
        exit(2);
    }

    ZeroMemory(&message, sizeof(message));
    message.lpszSubject = subject;
    message.lpszNoteText = body;
    originator.ulRecipClass = MAPI_ORIG;
    originator.lpszAddress = "Jeremy White <jwhite@codeweavers.com>";
    message.lpOriginator = &originator;

    if (file)
    {
        ZeroMemory(&file_desc, sizeof(file_desc));
        message.nFileCount = 1;
        message.lpFiles = &file_desc;
        file_desc.nPosition = (ULONG)-1;
        file_desc.lpszPathName = file;
        //file_desc.lpszPathName = strdup("C:\\");
        //file_desc.lpszFileName = strdup("\\blat");
    }

    if (to || bcc)
    {
        lpMapiRecipDesc rp;
        char *p;
        char *discard;

        if (to)
        {
            discard = strdup(to);
            for (p = strtok_r(discard, ",", &saveptr); p; p = strtok_r(NULL, ",", &saveptr))
                message.nRecipCount++;
            free(discard);
        }
        if (bcc)
        {
            discard = strdup(bcc);
            for (p = strtok_r(discard, ",", &saveptr); p; p = strtok_r(NULL, ",", &saveptr))
                message.nRecipCount++;
            free(discard);
        }

        rp = HeapAlloc(GetProcessHeap(), 0, message.nRecipCount * sizeof(*message.lpRecips));
        message.lpRecips = rp;
        if (to)
        {
            int sent = 0;
            for (p = strtok_r(to, ",", &saveptr); p; p = strtok_r(NULL, ",", &saveptr))
            {
                if (sent++ == 0)
                    rp->ulRecipClass = MAPI_TO;
                else
                    rp->ulRecipClass = MAPI_CC;
                rp->lpszAddress = p;
                rp++;
            }
        }

        if (bcc)
        {
            for (p = strtok_r(bcc, ",", &saveptr); p; p = strtok_r(NULL, ",", &saveptr))
            {
                rp->ulRecipClass = MAPI_BCC;
                rp->lpszAddress = p;
                rp++;
            }
        }

    }

    rc = mailer(0, 0, &message, 0, 0);
    printf("MAPISendMail returns %d\n", rc);

    return 0;
}

#if 0


void MailSender::AddFile( LPCTSTR file, LPCTSTR name )
{
    attachment a;
    a.path = file;
    if (!name)
        a.name = PathFindFileName(file);
    else
        a.name = name;
    m_Files.push_back(a);
}

bool MailSender::Send(HWND hWndParent, LPCTSTR szSubject)
int main(voi
{
    if (!m_hLib)
        return false;

    LPMAPISENDMAIL SendMail;
    SendMail = (LPMAPISENDMAIL) GetProcAddress(m_hLib, _T("MAPISendMail"));

    if (!SendMail)
        return false;

    vector<MapiFileDesc> filedesc;
    for (std::vector<attachment>::const_iterator ii = m_Files.begin(); ii!=m_Files.end(); ii++)
    {
        MapiFileDesc fileDesc;
        ZeroMemory(&fileDesc, sizeof(fileDesc));
        fileDesc.nPosition = (ULONG)-1;
        fileDesc.lpszPathName = (LPTSTR) ii->path.c_str();
        fileDesc.lpszFileName = (LPTSTR) ii->name.c_str();
        filedesc.push_back(fileDesc);
    }

    tstring subject;
    if (szSubject)
        subject = szSubject;
    else
    {
        for (std::vector<attachment>::const_iterator ii = m_Files.begin(); ii!=m_Files.end(); ii++)
        {
            subject += ii->name.c_str();
            if (ii+1 != m_Files.end())
                subject += ", ";
        }
    }

    MapiMessage message;
    ZeroMemory(&message, sizeof(message));
    message.lpszSubject = (LPTSTR) subject.c_str();
    message.nFileCount = filedesc.size();
    message.lpFiles = &filedesc[0];

    int nError = SendMail(0, (ULONG)hWndParent, &message, MAPI_LOGON_UI|MAPI_DIALOG, 0);

    if (nError != SUCCESS_SUCCESS && nError != MAPI_USER_ABORT && nError != MAPI_E_LOGIN_FAILURE)
        return false;

    return true;
}

#endif
